/*
 * This agent is paired with one or more www clients. It needs to be
 * set to return primary urls, otherwise it defaults to in-lines.
 * It expects to be the target of a FullTcpAgent.
 */
#ifndef lint
static char rcsid[] =
    "@(#) $Header: wwws.cc, Exp $ (LBL)";
#endif

#include "tcp-full.h"
#include "Tcl.h"
#include "random.h"
#include "trace.h"

class HTTPSrvrAgent : public TcpAppAgent {
 public:
	HTTPSrvrAgent();
	int command(int argc, const char*const* argv);
	void recv(Packet*, FullTcpAgent*);
protected:
	double now()  { Scheduler::instance().clock(); }
	int min_response_;	//number of bytes in min response file
	int max_response_;	//number of bytes in max response file
	int no_in_lines_;
	double shape_;		//the pareto distribution shape parameter
	static FILE* fp_;
};

FILE* HTTPSrvrAgent::fp_ = fopen("filesize.tr", "w");

static class HTTPSrvrClass : public TclClass {
public:
	HTTPSrvrClass() : TclClass("Agent/TcpApp/HTTPServer") {}
	TclObject* create(int, const char*const*) {
		return (new HTTPSrvrAgent());
	}
} class_https;

HTTPSrvrAgent::HTTPSrvrAgent() : TcpAppAgent(PT_NTYPE)
{
//	bind("min_byte_response", &min_response_);
//	bind("max_byte_response", &max_response_);
}

//should only be called when a full URL is received from a client
//need to make sure it goes to the right tcp connection

void HTTPSrvrAgent::recv(Packet* p, FullTcpAgent* tcp)
{
	int length = 0;
	if(((hdr_cmn*)p->access(off_cmn_))->size() < 350) {	//primary
		if(Random::uniform() < 0.1)
			length = 500;
		else {
			length = (int)Random::pareto(1000., 0.91);
			if(length > 15000)
				length = 15000; //in Bytes
		}
		tcp->advance(length, 0);
		fprintf(fp_, "%g %d %d\n", now(), 1, length);
	}
	else	{			//in-line
		for(int i=0; i< 3; i++) { //have to fix the 3 to be variable
			if(Random::uniform() < 0.1)
				length += 500;
			else {
				int l = (int)Random::pareto(1000., 1.24);
				if(l < 25000)
					length += l;
				else
					length += 25000; //in Bytes
			}
		}
		tcp->advance(length, 1);
		fprintf(fp_, "%g %d %d\n", now(), 0, length);
	}
}

/*
 * set to primary
 */
int HTTPSrvrAgent::command(int argc, const char*const* argv)
{
	if (argc == 2) {
		if (strcmp(argv[1], "no_in_lines") == 0) {
			no_in_lines_ = atoi(argv[2]);
			return (TCL_OK); 
		}
	}
	return (Agent::command(argc, argv));
}

